<?php
/**
 * definitions for the MyCode conversion done at install
 *
 * @category  MyBB Plugins
 * @package   YourCode
 * @author    Mark Vincent <admin@rantcentralforums.com>
 * @copyright 2012-2014 Mark Vincent
 * @license   http://opensource.org/licenses/gpl-license.php GNU Public License
 * @link      https://github.com/WildcardSearch/YourCode
 * @since     1.0
 */

// these are (formerly) internally cached MyCodes
$all_mycode = array(
	array(
		"title" => 'Format - Bold',
		"description" => 'MyBB Default Bold MyCode',
		"parse_order" => 10,
		"nestable" => false,
		"active" => true,
		"case_sensitive" => false,
		"single_line" => true,
		"multi_line" => false,
		"eval" => false,
		"callback" => false,
		"regex" => "\[b\](.*?)\[/b\]",
		"replacement" => '<span style="font-weight: bold;" class="mycode_b">$1</span>',
		"default_id" => 1,
	),
	array(
		"title" => 'Format - Underline',
		"description" => 'MyBB Default Underline MyCode',
		"parse_order" => 10,
		"nestable" => false,
		"active" => true,
		"case_sensitive" => false,
		"single_line" => true,
		"multi_line" => false,
		"eval" => false,
		"callback" => false,
		"regex" => "\[u\](.*?)\[/u\]",
		"replacement" => '<span style="text-decoration: underline;" class="mycode_u">$1</span>',
		"default_id" => 2,
	),
	array(
		"title" => 'Format - Italics',
		"description" => 'MyBB Default Italics MyCode',
		"parse_order" => 10,
		"nestable" => false,
		"active" => true,
		"case_sensitive" => false,
		"single_line" => true,
		"multi_line" => false,
		"eval" => false,
		"callback" => false,
		"regex" => "\[i\](.*?)\[/i\]",
		"replacement" => '<span style="font-style: italic;" class="mycode_i">$1</span>',
		"default_id" => 3,
	),
	array(
		"title" => 'Format - Strike-through',
		"description" => 'MyBB Default Strike-through MyCode',
		"parse_order" => 10,
		"nestable" => false,
		"active" => true,
		"case_sensitive" => false,
		"single_line" => true,
		"multi_line" => false,
		"eval" => false,
		"callback" => false,
		"regex" => "\[s\](.*?)\[/s\]",
		"replacement" => '<span style="text-decoration: line-through;" class="mycode_s">$1</span>',
		"default_id" => 4,
	),
	array(
		"title" => 'Copyright',
		"description" => 'MyBB Default Copyright MyCode',
		"parse_order" => 10,
		"nestable" => false,
		"active" => true,
		"case_sensitive" => false,
		"single_line" => false,
		"multi_line" => false,
		"eval" => false,
		"callback" => false,
		"regex" => "\(c\)",
		"replacement" => '&copy;',
		"default_id" => 5,
	),
	array(
		"title" => 'Trademark',
		"description" => 'MyBB Default Trademark MyCode',
		"parse_order" => 10,
		"nestable" => false,
		"active" => true,
		"case_sensitive" => false,
		"single_line" => false,
		"multi_line" => false,
		"eval" => false,
		"callback" => false,
		"regex" => "\(tm\)",
		"replacement" => '&#153;',
		"default_id" => 6,
	),
	array(
		"title" => 'Registered',
		"description" => 'MyBB Default Registered MyCode',
		"parse_order" => 10,
		"nestable" => false,
		"active" => true,
		"case_sensitive" => false,
		"single_line" => false,
		"multi_line" => false,
		"eval" => false,
		"callback" => false,
		"regex" => "\(r\)",
		"replacement" => '&reg;',
		"default_id" => 7,
	),
	array(
		"title" => 'URL - Simple #1',
		"description" => 'MyBB Default URL MyCode',
		"parse_order" => 10,
		"nestable" => false,
		"active" => true,
		"case_sensitive" => false,
		"single_line" => true,
		"multi_line" => false,
		"eval" => false,
		"callback" => true,
		"regex" => "\[url\]((?!javascript)[a-z]+?://)([^\r\n\"<]+?)\[/url\]",
		"replacement" => 'mycode_parse_url_callback1',
		"default_id" => 8,
	),
	array(
		"title" => 'URL - Simple #2',
		"description" => 'MyBB Default URL MyCode',
		"parse_order" => 10,
		"nestable" => false,
		"active" => true,
		"case_sensitive" => false,
		"single_line" => false,
		"multi_line" => false,
		"eval" => false,
		"callback" => true,
		"regex" => "\[url\]((?!javascript:)[^\r\n\"<]+?)\[/url\]",
		"replacement" => 'mycode_parse_url_callback2',
		"default_id" => 9,
	),
	array(
		"title" => 'URL - Complex #1',
		"description" => 'MyBB Default URL MyCode',
		"parse_order" => 10,
		"nestable" => false,
		"active" => true,
		"case_sensitive" => false,
		"single_line" => true,
		"multi_line" => false,
		"eval" => false,
		"callback" => true,
		"regex" => "\[url=((?!javascript)[a-z]+?://)([^\r\n\"<]+?)\](.+?)\[/url\]",
		"replacement" => 'mycode_parse_url_callback1',
		"default_id" => 10,
	),
	array(
		"title" => 'URL - Complex #2',
		"description" => 'MyBB Default URL MyCode',
		"parse_order" => 10,
		"nestable" => false,
		"active" => true,
		"case_sensitive" => false,
		"single_line" => true,
		"multi_line" => false,
		"eval" => false,
		"callback" => true,
		"regex" => "\[url=((?!javascript:)[^\r\n\"<]+?)\](.+?)\[/url\]",
		"replacement" => 'mycode_parse_url_callback2',
		"default_id" => 11,
	),
	array(
		"title" => 'Email - Simple',
		"description" => 'MyBB Default Email MyCode',
		"parse_order" => 10,
		"nestable" => false,
		"active" => true,
		"case_sensitive" => false,
		"single_line" => false,
		"multi_line" => false,
		"eval" => false,
		"callback" => true,
		"regex" => "\[email\](.*?)\[/email\]",
		"replacement" => 'mycode_parse_email_callback',
		"default_id" => 12,
	),
	array(
		"title" => 'Email - Complex',
		"description" => 'MyBB Default Email MyCode',
		"parse_order" => 10,
		"nestable" => false,
		"active" => true,
		"case_sensitive" => false,
		"single_line" => false,
		"multi_line" => false,
		"eval" => false,
		"callback" => true,
		"regex" => "\[email=(.*?)\](.*?)\[/email\]",
		"replacement" => 'mycode_parse_email_callback',
		"default_id" => 13,
	),
	array(
		"title" => 'Horizontal Rule',
		"description" => 'MyBB Default Horizontal Rule MyCode',
		"parse_order" => 10,
		"nestable" => false,
		"active" => true,
		"case_sensitive" => false,
		"single_line" => true,
		"multi_line" => false,
		"eval" => false,
		"callback" => false,
		"regex" => "\[hr\]",
		"replacement" => '<hr class="mycode_hr" />',
		"default_id" => 14,
	),
	array(
		"title" => 'Format - Color',
		"description" => 'MyBB Default Color MyCode',
		"parse_order" => 10,
		"nestable" => true,
		"active" => true,
		"case_sensitive" => false,
		"single_line" => true,
		"multi_line" => false,
		"eval" => false,
		"callback" => false,
		"regex" => "\[color=([a-zA-Z]*|\#?[\da-fA-F]{3}|\#?[\da-fA-F]{6})](.*?)\[/color\]",
		"replacement" => '<span style="color: $1;" class="mycode_color">$2</span>',
		"default_id" => 15,
	),
	array(
		"title" => 'Format - Size (string params)',
		"description" => 'MyBB Default Size MyCode',
		"parse_order" => 10,
		"nestable" => true,
		"active" => true,
		"case_sensitive" => false,
		"single_line" => true,
		"multi_line" => false,
		"eval" => false,
		"callback" => false,
		"regex" => "\[size=(xx-small|x-small|small|medium|large|x-large|xx-large)\](.*?)\[/size\]",
		"replacement" => '<span style="font-size: $1;" class="mycode_size">$2</span>',
		"default_id" => 16,
	),
	array(
		"title" => 'Format - Size (numeric params)',
		"description" => 'MyBB Default Size MyCode',
		"parse_order" => 10,
		"nestable" => true,
		"active" => true,
		"case_sensitive" => false,
		"single_line" => true,
		"multi_line" => false,
		"eval" => false,
		"callback" => true,
		"regex" => "\[size=([0-9\+\-]+?)\](.*?)\[/size\]",
		"replacement" => 'mycode_handle_size_callback',
		"default_id" => 17,
	),
	array(
		"title" => 'Format - Font',
		"description" => 'MyBB Default Font MyCode',
		"parse_order" => 10,
		"nestable" => true,
		"active" => true,
		"case_sensitive" => false,
		"single_line" => true,
		"multi_line" => false,
		"eval" => false,
		"callback" => false,
		"regex" => "\[font=([a-z0-9 ,\-_'\"]+)\](.*?)\[/font\]",
		"replacement" => '<span style="font-family: $1;">$2</span>',
		"default_id" => 18,
	),
	array(
		"title" => 'Format - Alignment',
		"description" => 'MyBB Default Font MyCode',
		"parse_order" => 10,
		"nestable" => true,
		"active" => true,
		"case_sensitive" => false,
		"single_line" => true,
		"multi_line" => false,
		"eval" => false,
		"callback" => false,
		"regex" => "\[align=(left|center|right|justify)\](.*?)\[/align\]",
		"replacement" => '<div style="text-align: $1;" class="mycode_align">$2</div>',
		"default_id" => 20,
	),
);

?>
